VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PPpage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'CLASS MODULE TO CREATE A PAGE FOR PRINTING OR PREVIEW.

'There should be nothing here that is specific to any particular project except that the
'error routines require a Public Boolean, 'canceled', in the main program.

' Philip Bolt, 2005, 2006
'Last updated: 19/11/6

Option Explicit
Option Compare Text

'Page numbers.
Private p_l_num As Integer          'The page LOCAL number.
Private p_a_num As Integer          'The page ABSOLUTE number.

'Page structure values.
Private t_marg As Integer           'Top margin.
Private h_marg As Integer           'Header margin.
Private b_marg As Integer           'Bottom margin.
Private f_marg As Integer           'Footer margin.
Private l_marg As Integer           'Left margin.
Private r_marg As Integer           'Right margin.
Private max_p_l As Integer          'Maximum page length, excluding bottom margins.
Private l_width As Integer          'Line width, excluding margins.
Private p_inc As Integer            'Page increment, 1 for ALL, 2 for ODD/EVEN.
Private p_d_s As Integer            'Printer width dead space.
Private ratio As Single             'Print/Preview scale factor.
Private nd As Integer               'End of text space indicator.
 
'Font data values.
Private l_ht As Integer             'Line height, (height of an 'I').
Private o_l_h As Integer            'Old line height after coded font size change.
Private f_name As String            'Font name.
Private sym_font As String          'Symbol' font name.
Private wd_font As String           'Windings' font name.
Private f_size As Single            'Font size.
Private f_bold As Boolean           'Bold.
Private f_italic As Boolean         'Italics.
Private f_underline As Boolean      'Underline.
Private f_strikethrough As Boolean  'Strikethrough.
Private s_f_n As String             'Stored font name.
Private s_f_z As Integer            'Stored font size.
Private s_f_b As Boolean            'Stored font Bold flag.
Private s_f_i As Integer            'Stored font Italic flag.
Private s_f_u As Integer            'Stored font Underline flag.
Private s_f_s As Integer            'Stored font StrikeThrough flag.
Private ss_set As Boolean           'Super or Subscript set flag.

'Print/Preview values.
Private pp_flg As Integer           'PRINT/PREVIEW flag, 0 = Print, 1 = Preview.
Private print_depth As Integer      'The printing depth on the page,
Private prntd As Integer            '(temporary printing depth storage).
Private p_zoom As Single            'The current zoom factor.

'Formated text values.
Private s_txt() As Integer          'Length of formated text sections.
Private spc_cnt() As Integer        'Number of spaces per text section.
Private max_spc As Integer          'Maximum number of spaces in any section.
Private coms As Integer             'Command count.
Private part_line As Integer        'Length of the existing part of a line.
Private tot_spc As Integer          'Total number of spaces in a line of text.
Private crit_len As Integer         'Critical length for a line.
Private back As Integer             'Back space size.
Private spc_sz As Single            'Size of a space in the current font.
Private com() As String             'Format command string array.
Private fl_char As String           'Text formating left character.
Private fr_char As String           'Text formating right character.
Private l_strt As Boolean           'Start of a new line flag for space removal.

'Graphics values.
Private pic As Object               'Graphics picture box.

Private page_exists As Boolean      'Page created flag.
Private setup As Boolean            'Object set up flag.

Private frm As Object               'Form object.
Private f_pic As Object             'Form picture object.

'Text alignment constants.
'VERTICAL
Const TA_TOP = 0            'The reference point will be on the top edge of the bounding
                            'rectangle of the text.
Const TA_BASELINE = 24      'The reference point will be on the baseline of the text.
Const TA_BOTTOM = 8         'The reference point will be on the bottom edge of the bounding
                            'rectangle of the text.
'HORIZONTAL
Const TA_LEFT = 0           'The reference point will be on the left edge of the bounding
                            'rectangle of the text.
Const TA_RIGHT = 2          'The reference point will be on the right edge of the bounding
                            'rectangle of the text.
Const TA_CENTER = 6         'The reference point will be horizontally centered along the
                            'bounding rectangle of the text.
'OTHER
Const TA_NOUPDATECP = 0     'Do not set the current point to the reference point.

Const TA_RTLREADING = 256   'Win 95/98 only:Display the text right-to-left (if the font is
                            'designed for right-to-left reading).
Const TA_UPDATECP = 1       'Set the current point to the reference point.

'Set text alignment API procedure.
Private Declare Sub SetTextAlign Lib "gdi32.dll" (ByVal hDC As Long, _
                                           ByVal wFlags As Long)
'Get Text alignment API function.
Private Declare Function GetTextAlign Lib "gdi32.dll" (ByVal hDC As Long) As Long

'Object constructor.
'Run when the object is created.

Private Sub Class_Initialize()

    If Not setup Then
        set_up
        setup = True
    End If
    
End Sub

'Procedure name:    Class_Terminate()
'Run when a page is terminated.

Private Sub Class_Terminate()

End Sub

'Routine to set up data when a Page object is created.
'Procedure name:    set_up

Sub set_up()
 
    f_name = "Arial"            'Set defaults for the font NAME,
    f_size = 12                 'and SIZE with
    f_bold = False              'no BOLD,
    f_italic = False            'no ITALIC,
    f_underline = False         'no UNDERLINE and
    f_strikethrough = False     'no STRIKETHROUGH.
    
    sym_font = "Symbol"         'Store the 'Symbol' font name.
    wd_font = "WingDings"       'Store the 'WingDings' font name.
    
    pp_flg = 1                  'Default to PREVIEW and
    p_zoom = 1                  'zoom off.
    
    part_line = 0               'Start with no part_lines.
    
    fl_char = "\"               'Store the text formating LEFT character.
    fr_char = "/"               'Store the text formating RIGHT character.
    
    Set frm = preview           'Define the form object and
    Set f_pic = frm.Picture2    'the Drawing Picture Box.
    Set pic = frm.Picture3      'Define the graphic picture object.
    
    back = 0                    'Initialise the backspace size.
    
    page_exists = True          'Flag that the page exists and
    ss_set = False              'clear the Super/Subscript,
    l_strt = False              'start of line and
    
End Sub

'Routine to set the page structure properties.
'Property Let name: set_sizes(opt as Integer, vlu as Variant)
'Parameter:         opt;    option number,
'                   vlu;    value.

Public Sub set_sizes(opt As Integer, vlu As Variant)

    Select Case opt
        Case 0
            max_p_l = vlu   'Maximum page length
        Case 1
            l_width = vlu   'Line width
        Case 2
            t_marg = vlu    'top margin.
        Case 3
            h_marg = vlu    'header margin,
        Case 4
            b_marg = vlu    'bottom margin.
        Case 5
            f_marg = vlu    'footer margin.
        Case 6
            l_marg = vlu    'left margin.
        Case 7
            r_marg = vlu    'right margin.
        Case 8
            p_inc = vlu     'page increment.
        Case 9
            p_d_s = vlu     'Printer page dead width space.
        Case 10
            ratio = vlu     'Print/Preview factor.
        Case Else
            MsgBox "Options must be in the range 0 to 9", vbOKOnly + vbCritical, "'Set_sizes' ERROR"
            canceled = True
    End Select
    
End Sub

'Routine to set the LOCAL and ABSOLUTE page numbers for the page.
'Procedure rather than Property Let used to allow optional parameter.

'Procedure name: page_numbers(lpn as Integer, Optional apn as Integer)
'Paramters:         lpn;    the LOCAL page number,
'                   apn;    the ABSOLUTE page number. This is optional. If missing. then
'                                                     defaults to the LOCAL.

Public Sub page_numbers(lpn As Integer, Optional apn As Integer = 0)
 
    p_l_num = lpn           'Assign the LOCAL page number

    If apn = 0 Then 'the ABSOLUTE page number is
        p_a_num = lpn           'either  a copy of the Local
    Else                        'or
        p_a_num = apn           'the parameter value.
    End If
 
End Sub

'Routine to read the page numbers of a specific page.
'Property name: page_number(opt As Integer)
'Parameter:     opt;    page selection option, 0 = LOCAL, 1 = ABSOLUTE.

Public Property Get page_number(opt As Integer)

    Select Case opt
        Case 0
            page_number = p_l_num 'LOCAL page number.
        Case 1
            page_number = p_a_num 'ABSOLUTE page number.
    End Select
    
End Property

'Routine to read the flag of a specific page.
'Property name: page_flag()

Public Property Get page_flag()

    page_flag = pp_flg  'The PRINT/PREVIEW flag.

End Property

'Set of property routines to allow access to page structure data.
'This is similar to the 'PP_pages' routine which should be used.
 
'Property name: page_data(opt)
'Parameter:     opt;    the item selection option.

Public Property Get page_data(opt As Integer)
 
    Select Case opt
        Case 0
            page_data = max_p_l 'Maximum page length
        Case 1
            page_data = l_width 'Line width
        Case 2
            page_data = t_marg  'top margin.
        Case 3
            page_data = h_marg  'header margin,
        Case 4
            page_data = b_marg  'bottom margin.
        Case 5
            page_data = f_marg  'footer margin.
        Case 6
            page_data = l_marg  'left margin.
        Case 7
            page_data = r_marg  'right margin.
        Case 8
            page_data = ratio   'Print/Preview scale factor.
        Case 9
            page_data = p_d_s   'Printer Width dead space.
        Case 10
            page_data = p_zoom  'The zoom factor.
        Case Else
            MsgBox "Options must be in the range 0 to 10", vbOKOnly + vbCritical, "'PPpage:page_data' ERROR"
            canceled = True
    End Select
    
End Property

'Routine to set the ZOOM factor for a page.
'Property name: set_zoom(zmf As Single)
'Parameter:     zmf;    the zoom factor.

Public Property Let set_zoom(zmf As Single)
 
    If pp_flg = 1 Then  'Set the zoom if Preview.
        p_zoom = zmf
    Else                'Otherwise,
        p_zoom = 1      'make it unity.
    End If
 
End Property

'Routine to set the page font data.
'All parameters are optional so that any one can be set alone.
'In the absence of a value, the current value is retained.
'Commas' are required to indicate missed parameters but don't use trailing ones.

'Procedure name:    font(Optional fnm As String = "", Optional fpt As Single,
'                        Optional fb As Variant, Optional fi As Variant,
'                        Optional fu As Variant, Optional fs As Variant)
'Parameters:        fnm;    the font name,
'                   fpt;    the point size,
'                   fb;     the Bold flag,
'                   fi;     the Italic flag,
'                   fu;     the Undeline flag,
'                   fs;     the StrikeThrough flag.

Public Sub font(Optional fnm As String = "", Optional fpt As Single, _
         Optional fb As Variant, Optional fi As Variant, _
         Optional fu As Variant, Optional fs As Variant)
 
    'STORE ANY CHANGES IN PARAMETERS
    If fnm <> "" Then
        f_name = fnm            'Set the font NAME,
    End If
        
    If IsNumeric(fpt) Then
        f_size = fpt                    'the SIZE
    End If
        
    If Not IsMissing(fb) Then
        f_bold = fb             'BOLD,
    End If
        
    If Not IsMissing(fi) Then
        f_italic = fi           'ITALIC,
    End If
        
    If Not IsMissing(fu) Then
        f_underline = fu        'UNDERLINE,
    End If
        
    If Not IsMissing(fs) Then
        f_strikethrough = fs    'STRIKETHROUGH.
    End If
     
    pp_font     'Re-set the font parameters.
    
End Sub

'Routine to set the font parameters in the light of the 'pp_flg' value.
'Both the PRINTER and Picture Box fonts are upgraded.
'Procedure name:    pp_font

Private Sub pp_font()

    With Printer.font               'Set the printer font parameters.
        .Name = f_name
        .Size = f_size
        .Bold = f_bold
        .Italic = f_italic
        .Underline = f_underline
        .Strikethrough = f_strikethrough
    End With

    l_ht = Printer.TextHeight("I")  'Store the height of a character, (line width), and
    spc_sz = Printer.TextWidth(" ")  'a space in the current font size.
    
    With f_pic.font          'Set the Picture Box font parameters.
        .Name = f_name
        .Size = f_size * ratio * p_zoom
        .Bold = f_bold
        .Italic = f_italic
        .Underline = f_underline
        .Strikethrough = f_strikethrough
    End With
 
End Sub

'Routine to return the current line height.
'Property name: line_height

Public Property Get line_height()

    line_height = l_ht  'Return the current line height.
    
End Property

'The item required is selected by the 'opt' parameter.
'Property name: font_data(opt)
'Parameter:     opt;    the item selection option.

Public Property Get font_data(opt As Integer)

    Select Case opt
        Case 0
            font_data = f_name          'Name,
        Case 1
            font_data = f_size          'point size,
        Case 2
            font_data = f_bold          'Bold
        Case 3
            font_data = f_italic        'Italic,
        Case 4
            font_data = f_underline     'Underline,
        Case 5
            font_data = f_strikethrough 'Strikethrough.
        Case Else
            MsgBox "Options must be in the range 0 to 5", vbOKOnly + vbCritical, "'font_data' ERROR"
            canceled = True
    End Select
    
End Property

'TEXT ADDITION ROUTINES.

'The PRINT/PREVIEW flag, 'pp_flg', is stored for the whole page.

'Routine to set the PRINT/PREVIEW flag for a page.
'This will CLEAR any data on the page to avoid the possibility of mixed
'Print and Preview formatted text.

'Property name: set_pp(ppf As Integer)
'Parameter:     ppf;    the PRINT/PREVIEW flag.

Public Property Let set_pp(ppf As Integer)

    If pp_flg = 0 Then  'Delete any existing text.
        Printer.KillDoc
    Else
        f_pic.Cls
    End If
    
    pp_flg = ppf    'Store the new value and
    pp_font         're-set the font parameters.
    
    print_depth = t_marg
    
End Property

'Routine to add SINGLE LINES of text to the printing area.
'The printing depth is returned to the calling routine via 'p_d'.
'The vertical position, 'vp', is an optional offset from the printing depth, 'p_d', which
'acts as a variable line width control. If ommmited, it is set to zero, (as will usually
'be the case).
'The alignment, 'alg', is also optional. If ommited, it is set to 0 = Left.

'If a line is too long for the page width and error message is displayed and the
'procedure ends.

'Procedure name:    add_text(tx, hp, tm, ByRef p_d, Optional vp, Optional alg)
'Parameters:        tx;     the TEXT,
'                   hp;     the HORIZONTAL position in twips,
'                   tm;     the LINE TERMINATOR flag.
'                   p_d;    the print depth, which is updated and returned.
'                   vp;     the VERTICAL offset in twips, defaults to zero,
'                   alg;    the alignment, 0 = Left, 1 = Centre, 2 = Right, defaults to 0.

Public Sub add_text(tx, ByRef hp, tm, ByRef p_d, Optional vp = 0, Optional alg = 0)
 
    Dim hpp, p_txt As Integer
 
    'Set the text display to LEFT justify and aligned with the BASE of letters.
    SetTextAlign preview.Picture2.hDC, TA_LEFT Or TA_BASELINE
    SetTextAlign Printer.hDC, TA_LEFT Or TA_BASELINE
    
    p_txt = Printer.TextWidth(tx)
    
    print_depth = p_d   'Take the current printing depth from the parameter.
    
    If hp = 0 Then  'Set the initial alignment.
        Select Case alg                 'All alignments include the left margin.
            Case 0 'is LEFT aligned.
                hpp = hp + l_marg
            Case 1 'is CENTRE aligned.
                hpp = l_marg + centre(tx) + hp
            Case 2 'is RIGHT aligned.
                hpp = l_marg + l_width - p_txt - hp
            Case Else 'is an error, (e1).
                MsgBox "Alignment codes must be 0, 1 or 2", vbOKOnly + vbCritical, "'add_text' ERROR"
                canceled = True
        End Select
    Else
        hpp = hp
    End If
 
    If Not canceled Then
        If p_txt + hpp > l_marg + l_width Then 'Text runs over the end of the line, (e2).
            MsgBox "The text is longer than a line.", vbOKOnly + vbCritical, "'add_text' ERROR"
            canceled = True
        Else
            If pp_flg = 0 Then 'print.
                hpp = hpp + p_d_s   'Add in the Printer Width dead space.
                Printer.CurrentX = hpp              'Position text at specified X position and
                Printer.CurrentY = print_depth + vp 'Y position but
                Printer.Print tx;                   'don't terminate the print line.
                If tm Then
                    Printer.Print                   'Terminate the line, if flagged.
                    print_depth = Printer.CurrentY  'Store the vertical position.
                    hp = 0                          'Reset the horizontal position.
                Else
                    hp = Printer.CurrentX           'Return the horizontal position via the parameter.
                End If
            Else    'Preview
                f_pic.CurrentX = hpp * ratio * p_zoom                'Position text at specified X position and
                f_pic.CurrentY = (print_depth + vp) * ratio * p_zoom 'Y position but
                f_pic.Print tx;                                      'don't terminate the print line.
                p_text tx, hpp, 0
                If tm Then
                    f_pic.Print                                     'Terminate the line, if flagged.
                    print_depth = f_pic.CurrentY / (ratio * p_zoom) 'Store the vertical position.
                    hp = 0                                          'Reset the horizontal position.
                Else
                    hp = f_pic.CurrentX / (ratio * p_zoom)  'Return the horizontal position via the parameter.
                End If
            End If
 
            p_d = print_depth   'Return the current print depth via the parameter.
        End If 'error 2
    End If 'error 1
   
End Sub

'Routine to add FORMATTED text to the printing area.
'The updated printing depth is returned to the calling routine via 'p_d'.
'The vertical position, 'vp', is an optional offset from the printing depth, 'p_d', which
'acts as a variable line width control. If ommmited, it is set to zero, (as will usually
'be the case).
'The alignment, 'alg', is also optional. If ommited, it is set to 0 = Left.

'This is NOT a word processor.
'Text already displayed is NOT modified by the addition of extra text. e.g. changing the
'margins doesn't effect the existing display.

'Procedure name:    add_f_text(tx, hp, tm, ByRef p_d, Optional vp, Optional alg)
'Parameters:        tx;     the TEXT,
'                   hp;     the HORIZONTAL position in twips,
'                   tm;     the LINE TERMINATOR flag.
'                   p_d;    the PRINT DEPTH, for return to the main program,
'                   alg;    the ALIGNMENT, 0 = Left, 1 = Centre, 2 = Right, 3 = Justified,
'                           optional, default = 0, (Left justified),
'                   p_l;    the PART LINE length, optional, default = none, (0),
'                   vp;     the VERTICAL position in twips, (this is an offset for EACH
'                           line), optional; default = 0, standard line spacing.

Public Sub add_f_text(tx, hp, tm, ByRef p_d, Optional alg = 0, Optional p_l = 0, _
                      Optional vp = 0)
 
    Dim hpp, p_txt, j, sect, spac, st_sc, st_sp, hph, vpv, txt_len As Integer
    Dim txx As String
 
    'Set the text display to LEFT justify and aligned with the BASE of letters.
    SetTextAlign f_pic.hDC, TA_LEFT Or TA_BASELINE
    SetTextAlign Printer.hDC, TA_LEFT Or TA_BASELINE
    
    If alg > 3 Or alg < 0 Then
        MsgBox "Alignment codes are 0, 1, 2 or 3", vbOKOnly + vbCritical, "'add_j_text' ERROR"
        canceled = True
    End If
 
    If Not canceled Then
        print_depth = p_d   'Take the current printing depth from the parameter.
        part_line = p_l     'Used to return the current line position if not terminated.
 
        parse_text tx       'Split the text into its command sections.

        p_txt = text_len1   'Find the overall text length.
        re_set_font
 
        If canceled Then Exit Sub 'Quit if ERROR.
        
        crit_len = l_width - hp
 
        sect = 0: spac = 0      'Initialise the text section and space values.
        Do Until p_txt <= crit_len  'Loop until the remaining text fits on a line.
            st_sc = sect: st_sp = spac  'Store the start section and space number.
            txt_len = find_split(crit_len, sect, spac) 'Find the position to split the text.
 
            If canceled Then Exit Sub 'Quit if ERROR.
            
            hph = aline(alg, hp, txt_len, part_line) 'Set the horizontal value for the required alignment.
 
            If alg = 3 Then
                print_j_text txt_len, hph, vp, st_sc, st_sp, sect, spac
            Else
                print_text hph, vp, st_sc, st_sp, sect, spac   'print this length unjustified.
            End If
 
            part_line = 0   'After the first one, each part has a full line.
            p_txt = text_len2(sect, spac)   'Find the new length of the text.
            If canceled Then Exit Sub   'Exit on text length routine error.
            print_depth = print_depth + l_ht
            terminate
        Loop
    
        If p_txt > 0 Then 'there's still some left to print, so
            'print the final part.
            hph = aline(alg, hp, p_txt, part_line) 'Set the horizontal value for the required alignment.
            st_sc = sect: st_sp = spac  'Store the start section and space number and
            sect = coms + 1: spac = 0   'set the end values for the end of the text.
 
            print_text hph, vp, st_sc, st_sp, sect, spac   'Print this section.
  
            If tm Or alg = 1 Or alg = 2 Then 'There can be no part line for RIGHT or
                terminate       'CENTRE justification, so the line is terminated.
                part_line = 0   'If the line is terminated, there is no part line.
            Else
                part_line = hph     'Store the current length of the part line for future use.
            End If 'terminated.
        End If 'some left to print.
       
        p_d = print_depth   'Return the current print depth
        p_l = part_line     'and the part line value via the parameters.
    End If 'not canceled
 
End Sub

'Routine to terminate a print/display line.
'Procedure name:    terminate

Public Sub terminate()

    If pp_flg = 0 Then
        Printer.Print                   'Terminate the line, if flagged.
        print_depth = Printer.CurrentY  'Store the vertical position.
    Else
        f_pic.Print                                      'Terminate the line, if flagged.
        print_depth = f_pic.CurrentY / (ratio * p_zoom)  'Store the vertical position.
    End If
        
End Sub

'Routine to divide up a string into TEXT and FORMATING characters.
'Function name: parse_text(ttx)
'Parameter:     ttx;    the text string.

Private Sub parse_text(ttx)

    Dim pt, l_m, r_m As Integer
 
    pt = 1                  'the character pointer,
    coms = 0                'the command string count and
    ReDim com(1, coms + 1)  'dimension the command storage array.
    
    l_m = InStr(pt, ttx, fl_char)  'Find the first left marker, '\'.
    If Mid(ttx, l_m + 1, 1) = fl_char Then 'Next character is '\', so
        ttx = shorten_text(ttx, l_m)        'remove one of the '\' characters and
        Do Until Mid(ttx, l_m + 1, 1) <> fl_char 'loop
            l_m = l_m + 1                   'until a non-'\' character is found.
        Loop
        l_m = InStr(l_m + 1, ttx, fl_char) 'find the next left marker, '\'.
    End If
 
    Select Case l_m
        Case Is > 1 'there's text before the first code block, so
            com(0, 0) = ""                  'store no code and
            com(1, 0) = Left(ttx, l_m - 1)  'the initial section of text.
            coms = 1
        Case 0  'there aren't any formating codes, so
            com(0, 0) = ""  'store no code and
            com(1, 0) = ttx 'all the text.
    End Select
  
    If l_m > 0 Then
        Do Until l_m = 0
            If l_m <> 0 Then 'there may be formating commands, so
                r_m = InStr(l_m, ttx, fr_char)   'Find a right marker , '/'.
                If r_m <> 0 And r_m <> l_m + 1 Then 'there are  formating commands, so
                    ReDim Preserve com(1, coms + 1)                 'redimension the command array,
                    com(0, coms) = Mid(ttx, l_m + 1, r_m - l_m - 1) 'store the commands and
                    pt = r_m + 1                    'Update the pointer and
                    l_m = InStr(pt, ttx, fl_char)   'find the next left marker, '\'.
                    If l_m = 0 Then 'there isn't another left marker, so
                        com(1, coms) = Right(ttx, Len(ttx) - r_m) 'store the rest of the text.
                    Else                                        'or
                        If Mid(ttx, l_m + 1, 1) = fl_char Then 'Next character is '\', so
                            ttx = shorten_text(ttx, l_m)        'remove one of the '\' characters and
                            Do Until Mid(ttx, l_m + 1, 1) <> fl_char    'loop
                                l_m = l_m + 1                   'until a non-'\' character is found.
                            Loop
                            l_m = InStr(l_m + 1, ttx, fl_char) 'find the next left marker, '\'.
                        End If
                        com(1, coms) = Mid(ttx, r_m + 1, l_m - r_m - 1)     'the next section of text.
                    End If
                    coms = coms + 1     'Increment the command count.
                End If
            End If
        Loop
        coms = coms - 1 'Adjust the command count.
    End If
 
    ss_set = False      'clear the Super/Subscript set flag.

End Sub

'Routine to remove a code indicator from the text to leave a single character.
'Function name: shorten_text(pt)
'Parameter:     pt, the position of the character to be removed.

Private Function shorten_text(txtx, pt)

    Dim ln As Integer
    
    ln = Len(txtx)  'Store the length of the string.
    
    shorten_text = Left(txtx, pt - 1) + Right(txtx, ln - pt)
    
End Function

'Routine to find the TOTAL length of the text.
'This, the intitial version, also finds the spaces in each 'com(1, )' section of text.
'Function name: text_len1()

Private Function text_len1()

    Dim t_l, j, k, n_spc As Integer 'Dimension the variables and
    t_l = part_line                 'initialise the text length counter and
    max_spc = 1                     'the maximum number of spaces.
 
    ReDim s_txt(coms), spc_cnt(coms, max_spc) 'Dimension the text and spaces storage arrays.
    
    prntd = print_depth 'Store the printing depth as it can change in 'set_font'.
    
    For j = 0 To coms
        If com(1, j) = "" Then 'there is no text, so
            MsgBox "There's no text in section " + CStr(j), vbOKOnly, "TEXT ERROR"
            canceled = True 'Set the Canceled flag,
            text_len1 = 0   'return zero and
            Exit Function   'exit.
        End If
        
        set_font com(0, j), j   'Set up the font and
        s_txt(j) = Printer.TextWidth(com(1, j)) - back  'Store the length of this section.
        t_l = t_l + s_txt(j)                    'add it to the formated length.
 
        n_spc = 1       'Initialise the space count.
        For k = 1 To Len(com(1, j))     'Check this section for spaces.
            If Mid(com(1, j), k, 1) = " " Then  'this is a space, so
                spc_cnt(j, n_spc) = k           'store its position in the section and
                n_spc = n_spc + 1               'increment the array pointer.
                If n_spc > max_spc Then         'If the maximum is exceeded,
                    max_spc = n_spc             'redefine the maximum and
                    ReDim Preserve spc_cnt(coms, max_spc)   'redimension the array.
                End If
            End If
        Next 'character
        spc_cnt(j, 0) = n_spc - 1   'Store the number of spaces in this section.
    Next 'section.
    
    print_depth = prntd 'Reset the current printing depth.
 
    text_len1 = t_l
    
End Function

'Routine to find the 'spliting' space in a text string and
'to return the length of the text for alignment.
'Function name:    find_split(cl, sct, spp)
'Parameter:         cl;     the length that will fit on a line,
'                   sct;    the starting section and
'                   spp;    space number, (0 = start at begining of section).
'NB                         The parameters are used to return the section and space
'                           at which to make the split.

Private Function find_split(ByVal cl, ByRef sct, ByRef spp)
 
    Dim t_l, strt, j, k, scn, spn, pt, sp_n, n_sp, sp, st As Integer
    Dim tx As String
 
    prntd = print_depth 'Store the printing depth as it can change in 'set_font'.
    
    If part_line = 0 Then
        strt = 0
    Else
        strt = part_line - l_marg   'Initialise the length,
    End If
    t_l = strt
 
    scn = sct           'store the starting section number,
    spn = spp           'the starting space number,
    tot_spc = 0         'initialise the total space count and
    l_strt = True       'Set the Start of new line flag.
 
    'Add in section lengths until limit is exceeded.
    Do Until t_l > cl Or scn > coms
        If spn = 0 Then                         'All the block is to be included, so
            t_l = t_l + s_txt(scn)              'store the length of all of this section and
            tot_spc = tot_spc + spc_cnt(scn, 0) 'add in the space count.
            If l_strt Then    'Remove any spaces at the start of the line.
                pt = 1
                Do Until Left(com(1, scn), pt) <> " "   'Loop until a non-space found.
                    pt = pt + 1
                Loop
                tot_spc = tot_spc - pt + 1  'Update the total number of spaces and
                l_strt = False              'clear the Start flag.
            End If
        Else                                    'or
            t_l = t_l + get_length(scn, spn)    'store the part from 'spp' onwards.
        End If
        scn = scn + 1
        spn = 0
    Loop
 
    If t_l > cl Then  'the length was greater than that allowed.
        scn = scn - 1   'Store final section number.
        'Back-track one section
        If scn > sct Then 'the first section has been dealt with.
            t_l = t_l - s_txt(scn)              'Reduce the length by the last section,
            tot_spc = tot_spc - spc_cnt(scn, 0) 'deduct in the space count for this section and
            spn = 0                             'set the space count for the start of the section.
            pt = 1  'Check for spaces at the end of the previous section.
            store_font 0    'Store the current font settings, then
            set_font com(0, scn - 1), scn - 1   'set up the font for the previous section.
 
            Do Until Right(com(1, scn - 1), pt) <> " " 'Loop until a non-space found.
                pt = pt + 1
            Loop
            tot_spc = tot_spc - pt + 1      'Update the total number of spaces and
            t_l = t_l - (pt - 1) * spc_sz   'the total length by removing space sizes.
            store_font 1    'Restore the previous font settings.
        Else            'or we're still in the first section, so
            t_l = strt  'reset to the start value,
            tot_spc = 0 'zero the total spaces and
            spn = spp   'set the space count to the parameter value.
        End If
        
        'Check through the spaces in the next section to fill any available space on the line.
        store_font 0    'Store the current font data.
        set_font com(0, scn), scn   'Set up the fonts and
        t_l = t_l - back            'take account of any back space.
        If Not canceled Then
            n_sp = spc_cnt(scn, 0)  'Store the number of spaces in this section.
            If n_sp > 0 Then    'There are spaces.
                If spn = 0 Then 'If this is the start of the section,
                    sp_n = 1        'start at the first space and
                    st = 1          'set the start character to the first.
                Else                'Otherwise,
                    st = spc_cnt(scn, spp) + 1 'the start character to one after the parm. space.
                    sp_n = spn + 1  'set the first space to the parm. value plus 1 and
                End If
                sp = spc_cnt(scn, sp_n)    'Store the space position.
 
                Do Until t_l > cl Or sp_n > n_sp
                    tx = Mid(com(1, scn), st, sp - st)  'Get the part string and
                    t_l = t_l + Printer.TextWidth(tx)  'add in its length.
                    sp_n = sp_n + 1         'Increment the space count and
                    tot_spc = tot_spc + 1   'the total number of spaces.
                    st = sp                 'Set the new start position and
                    sp = spc_cnt(scn, sp_n) 'store the  position of the next space.
                Loop
 
                sct = scn       'Store the section and
                If t_l > cl Or Trim(tx) = "" Then 'there is overshoot or a final space, so
                    spp = sp_n - 2  'decrement the last space number for return, (-1 to step
                                    'back 1 and another -1 to remove the last increment).
                    
                    'Remove the extra word to find text length for alignment start.
                    t_l = t_l - Printer.TextWidth(tx)
                    tot_spc = tot_spc - 1   'Decrement the total spaces for the word removed.
                Else
                    spp = sp_n - 1  'Just decrement the last space count.
                End If
            Else            'Otherwise,
                sct = scn   'the split must come at the end of the previous section.
                spp = 0     '(0 on EXIT flags for the PREVIOUS block to the end).
 
                'Check for trailing spaces on previous block.
                pt = 1  'Check for spaces at the end of the previous section.
                store_font 0    'Store the current font settings.
                set_font com(0, scn - 1), scn - 1 'Set up the font for the previous section and
                t_l = t_l - back                  'take account of any back space.
                Do Until Right(com(1, scn - 1), pt) <> " " 'Loop until a non-space found.
                    pt = pt + 1
                Loop
                tot_spc = tot_spc - pt + 1      'Update the total number of spaces and
                t_l = t_l - (pt - 1) * spc_sz   'the total length by removing space sizes.
                store_font 1    'Restore the font settings.
            End If
        End If 'set_font' canceled.
    Else
        sct = scn
        spp = 0
    End If
    
    store_font 1        'Replace the stored font data.
    print_depth = prntd 'Reset the current printing depth.
    
    find_split = t_l

End Function

'Routine to find the length of the LAST part of a section.
'Function name: get_length(sc, sp)
'Parameter:     sc;     The starting section and
'               sp;     space, (0 = start at begining of section).

Private Function get_length(sc, sp)

    Dim pt As Integer
    Dim p_txt As String
    
    p_txt = Right(com(1, sc), Len(com(1, sc)) - spc_cnt(sc, sp))

    tot_spc = tot_spc + count_spaces1(p_txt, 1) 'Total the spaces.
    
    If l_strt Then    'Remove any leading spaces
        pt = 1
        Do Until Left(p_txt, pt) <> " "   'Loop until a non-space found.
            pt = pt + 1
        Loop
        tot_spc = tot_spc - pt + 1  'Update the total number of spaces and
        l_strt = False              'clear the Start flag.
    End If
 
    get_length = Printer.TextWidth(p_txt)

End Function

'Routine to count the spaces in a section of text.
'Function name: count_spaces1(txst)
'Parameter:     txst; the text string.
'               stp;    the starting position.

Private Function count_spaces1(txst, stp)

    Dim s_c, s_p1, s_p2 As Integer

    s_c = 0     'Initialise the space count and
    s_p1 = stp  'the starting position.
    s_p2 = InStr(s_p1, txst, " ") 'Find the position of the first space.
    
    Do Until s_p2 = 0
        s_c = s_c + 1       'Count the space,
        s_p1 = s_p2 + 1     'reset the start position and
        s_p2 = InStr(s_p1, txst, " ") 'find the position of the next space.
    Loop
    
    count_spaces1 = s_c
    
End Function

'Routine to set the horizontal position for the start of a line
'in terms of the required alignment and the text length.
'Function name: aline(al, h, txt_l)
'Parameters:    al;     the alignment code,0, 1, or 2, (code 3, [Fully justified], is dealt
'                       with by its own printing routine, 'print_j_text').
'               h;      the horizontal offset,
'               txt_l;  the length of the formated text,
'               prt_l;  the partial line value.

Private Function aline(al, h, txt_l, Optional prt_l = 0)

    Dim hpl As Integer
 
    hpl = 0     'Set default return value.
    
    Select Case al 'All alignments include the left margin.
        Case 0, 3 'is LEFT or FULLY JUSTIFIED.
            If prt_l > 0 Then
                hpl = prt_l + spc_sz    'Use in the partial line value with a single space.
            Else                        'Otherwise,
                hpl = l_marg + h        'Start at the left margin plus any offset.
            End If
        Case 1 'is CENTRED.
            hpl = l_marg + h + (crit_len - txt_l) / 2
        Case 2 'is RIGHT.
            hpl = l_marg + h + crit_len - txt_l
    End Select
   
    If pp_flg = 0 Then      'this is for Printing, so
        hpl = hpl + p_d_s   'add in the Printer Width dead space.
    End If
                
    aline = hpl

End Function

'Routine to print a length of text, other than fully formatted.
'Procedure name:    print_text(hhp, vvp, sc1, sp1, sc2, sp2)
'Parameters:        hhp;    the horizontal position,
'                   vvp;    the vertical offset,
'                   sc1;    start section,
'                   sp1;    start space number,
'                   sc2;    end section,
'                   sp2;    end space number.

Private Sub print_text(hhp, vvp, sc1, sp1, sc2, sp2)
 
    Dim done As Boolean, txxt As String, strt, j As Integer   'Printing termination flag.
 
    set_font com(0, sc1), sc1   'Set up the font for the first section.
    
    'FIRST part
    Select Case True
        Case sp1 = 0 And sc2 > sc1  'All of the first section is needed, so
            txxt = com(1, sc1)      'store the whole section.
            done = False
        Case sp1 > 0 And sc2 > sc1  'The LAST part of the first section is needed, so
            strt = spc_cnt(sc1, sp1) 'store the section space position and
            txxt = Right(com(1, sc1), Len(com(1, sc1)) - strt)  'the part-section.
            done = False
        Case sp1 = 0 And sc2 = sc1  'Only the FIRST part of the section is needed, so
            txxt = Left(com(1, sc1), spc_cnt(sc1, sp2) - 1)   'take the left-hand part.
            done = True
        Case sc2 = sc1 And sp2 > 0 'Only part of the section is needed, so
            txxt = Mid(com(1, sc1), spc_cnt(sc1, sp1), spc_cnt(sc1, sp2) - spc_cnt(sc1, sp1)) 'store the part required.
            done = True
    End Select

    p_text LTrim(txxt), hhp, vvp   'Print the text.
 
    'MIDDLE part, if any.
    If Not done Then 'there is still more to do!
        If sc2 - sc1 > 1 Then  'a number of whole sections are needed, so
            For j = sc1 + 1 To sc2 - 1
                set_font com(0, j), j   'set up the font for the section.
                p_text com(1, j), hhp, vvp  'print it.
            Next
        End If
 
        'FINAL part, sc2, if any.
        If sp2 > 0 Then
            set_font com(0, sc2), sc2
            txxt = Left(com(1, sc2), spc_cnt(sc2, sp2))    'Take the left-hand section.
            p_text txxt, hhp, vvp   'Print the final part.
        End If
    End If
 
End Sub

'Routine to print a length of fully justified text.
'tot_spc' holds the number of spaces for the section.
'Procedure name:    print_j_text(tx_l, hhp, vvp, sc1, sp1, sc2, sp2)
'Parameters:        tx_l    the length of the text block,
'                   hhp;    the horizontal position,
'                   vvp;    the vertical offset,
'                   sc1;    start section,
'                   sp1;    start space number,
'                   sc2;    end section,
'                   sp2;    end space number.

Private Sub print_j_text(tx_l, hhp, vvp, sc1, sp1, sc2, sp2)
 
    Dim j, php As Integer
    Dim a_s As Single
    Dim done As Boolean
 
    a_s = ((crit_len - tx_l) / tot_spc) 'Additional length to be added per space.
 
    set_font com(0, sc1), sc1   'Set up the font for the first section.
'MsgBox com(0, sc1)
    l_strt = True 'Set the Start of new line flag.
 'MsgBox sc1 & "/" & sp1 & "/" & sc2 & "/" & sp2
  'MsgBox Printer.font.Italic & "PJT"
   'FIRST part
    Select Case True
        Case sp1 = 0 And sc2 > sc1  'All of the first section is needed, so
            p_j_text hhp, vvp, a_s, sc1, 0, 0
            done = False
        Case sp1 > 0 And sc2 > sc1  'The LAST part of the first section is needed, so
'MsgBox sc1 & "/" & sp1 & "/" & sc2 & "/" & sp2
            p_j_text hhp, vvp, a_s, sc1, sp1, 0
            done = False
        Case sp1 = 0 And sc2 = sc1  'Only the FIRST part of the section is needed, so
            p_j_text hhp, vvp, a_s, sc1, 0, sp2
            done = True
        Case sc2 = sc1 And sp2 > 0 'Only part of the section is needed, so
            p_j_text hhp, vvp, a_s, sc1, sp1, sp2
            done = True
    End Select
 
    'MIDDLE section, if any.
    If Not done Then
        If sc2 - sc1 > 1 Then  'a number of whole sections are needed, so
            For j = sc1 + 1 To sc2 - 1
                set_font com(0, j), j           'Set up the font for the section and
'MsgBox com(0, j)
                p_j_text hhp, vvp, a_s, j, 0, 0 'print it.
            Next
        End If
 
        'FINAL part, sc2, if any.
        If sp2 > 0 Then
            set_font com(0, sc2), sc2
            p_j_text hhp, vvp, a_s, sc2, 0, sp2 'Print the final part.
        End If
    End If
 
End Sub

'Routine to print a section of text.
'Procedure name:    p_text(txx, hh, vv)
'Parameters:        txx;    the text string,
'                   hh;     the horizontal position,
'                   vv;     the vertical offset.

Private Sub p_text(txx, ByRef hh, Optional vv = 0)
  
    'Dim retval As Long  ' return value
 'If txx = "section" Then MsgBox Printer.font.Italic
    If pp_flg = 0 Then 'print.
        Printer.CurrentX = hh - back             'Position text at specified X position and
        Printer.CurrentY = print_depth + vv 'Y position.
        Printer.Print txx;
        hh = Printer.CurrentX
    Else
        f_pic.CurrentX = (hh - back) * ratio * p_zoom               'Position text at specified X position and
        f_pic.CurrentX = (hh - back) * ratio * p_zoom
 
        f_pic.CurrentY = (print_depth + vv) * ratio * p_zoom 'Y position.
        f_pic.Print txx;
        hh = (f_pic.CurrentX) / (ratio * p_zoom)
    End If
 
    back = 0

End Sub

'Routine to print a section of fully justified text.
'Procedure name:    p_j_text(ByRef hh, vv, a_s, sc, s1, s2)
'Parameters:        hh;     the horizontal position,
'                   vv;     the vertical offset.
'                   a_s;    additional length per space,
'                   sc;     section concerned,
'                   s1;     starting space, 0 = start of section,
'                   s2;     last space, 0 = end of section.

Private Sub p_j_text(ByRef hh, vv, a_s, sc, s1, s2)
 'MsgBox Printer.font.Italic
    Dim s_s1, s_s2, spp, php As Integer
    Dim tx As String
 
    If s1 = 0 Then
        s_s1 = 1                    'Start at the first character
    Else                            'or
        s_s1 = spc_cnt(sc, s1) + 1  'the character AFTER the specified space.
    End If
'If sc = 2 Then MsgBox s_s1
 
    If s2 <> 0 Then
        s_s2 = spc_cnt(sc, s2)  'Set the value to the last space position
    Else                        'or
        s_s2 = Len(com(1, sc))  'the length of the section.
    End If
'If sc = 2 Then MsgBox s_s2
    php = hh    'Store the horizontal position locally.
    
    spp = 1     'Dummy space position.
     
    If spc_cnt(sc, 0) = 0 Then 'there are no spaces in this block, so
        If l_strt Then
            p_text LTrim(com(1, sc)), php, vv  'Trim and print the whole section.
            l_strt = False 'Clear the Start flag.
        Else
            p_text com(1, sc), php, vv  'Print the whole section.
        End If
        php = php + a_s             'Increment the horizontal position.
    Else
        Do Until s_s1 >= s_s2 Or spp = 0
            spp = InStr(s_s1, com(1, sc), " ")  'Find the next space,
'If sc = 2 Then MsgBox spp
            If spp <> 0 Then
                If l_strt Then
                    tx = LTrim(Mid(com(1, sc), s_s1, spp - s_s1 + 1))  'Trim and print the section.
                    l_strt = False 'Clear the Start flag.
                Else
                    tx = Mid(com(1, sc), s_s1, spp - s_s1 + 1) 'Cut out the word.
                End If

                If tx <> "" Then
                    p_text tx, php, vv  'print it.
                End If
                s_s1 = spp + 1      'Reset the pointer for the next word and
                php = php + a_s     'increment the horizontal position.
            End If
    'MsgBox tx
        Loop

        If s_s1 <= s_s2 Then 'there's still a word left, so
            If l_strt Then
                tx = LTrim(Mid(com(1, sc), s_s1, s_s2 - s_s1 + 1))
            Else
                tx = Mid(com(1, sc), s_s1, s_s2 - s_s1 + 1) 'isolate the word and
            End If
'If sc = 2 Then MsgBox tx
'If tx = "section" Then MsgBox Printer.font.Italic
            p_text tx, php, vv  'print it.
            php = php + a_s
        End If
    End If

    hh = php    'Update the horizontal position.
 
End Sub

'Routine to find the length of text.
'Function name: text_len2(sct, spp)
'Parameter:     sct;    the starting section and
'               spp;    space, (0 = start at begining of section).

Private Function text_len2(sct, spp)
 
    Dim t_l, j, k As Integer, txx As String 'Dimension the variables and
    t_l = 0     'initialise the text length counter.
    
    prntd = print_depth 'Store the printing depth as it can change in 'set_font'.
 
    If sct <= coms Then 'there's still some text to process.
        If com(1, 0) = "" Then 'there isn't any text, so
            MsgBox "There isn't any text in the first block!", vbOKOnly, "TEXT ENTRY ERROR"
            canceled = True
        Else
            If com(0, sct) <> "" Then   'there are commands for this section, so
                set_font com(0, sct), sct   'set up the fonts and
                t_l = t_l - back            'take account of any back space.
            End If
            If spp = 0 Then
                t_l = t_l + Printer.TextWidth(com(1, sct))  'Store the length of all this section.
            Else                    'Store the length from space 'spp' onwards.
                txx = Mid(com(1, sct), spc_cnt(sct, spp))
                t_l = t_l + Printer.TextWidth(txx)
            End If
 
            If sct + 1 < coms Then 'there are some sections left, so
                For j = sct + 1 To coms
                    If com(0, j) <> "" Then 'there are commands, so
                        set_font com(0, j), j   'set up the fonts and
                        t_l = t_l - back        'take account of any back space.
                    End If
                    t_l = t_l + Printer.TextWidth(com(1, j))
                Next
            End If
        End If 'no error.
    End If 'still some text to process.
    
    print_depth = prntd 'Reset the current printing depth.
 
    text_len2 = t_l

End Function

'Routine to add a graphic to the page.

'The position values must be given in twips from LEFT and TOP margins but the graphic
'WIDTH and HEIGHT can be given either in twips or as a FRACTION of the page width and height.

'The default value of zero for the width means that the source width is maintained.
'The default value of zero for the height means that the proportion of the graphic is
'maintained. i.e. zeroes for 'gw' and 'gh' mean that the graphic is displayed at its
'original size.
'If 'hh' is defaulted to -1, the graphic will be CENTRED on the page.
'If 'vv' is defaulted to zero, the graphic is printed one line space below the current
'PAGE DEPTH.

'Procedure name:    add_graphic(g_path As String, Optional gw As Single = 0,
'                   Optional gh As Single = 0, Optional hh As Integer = -1,
'                   Optional vv As Integer = 0)

'Parameters:        path;       graphic file name plus full path,
'                   gw;         graphic width in twips or as a fraction of the page width,
'                   gh;         graphic height in twips or as a fraction of the page height,
'                   hh;         relative horizontal position or default, (-1), to centre,
'                   vv;         relative vertical position or default, (0), for current
'                               depth plus a single line space.

Public Sub add_graphic(g_path As String, Optional gw As Single = 0, Optional gh As Single = 0, _
                       Optional hh As Integer = -1, Optional vv As Integer = 0)

    Dim a_dp, s_w, s_h, hp, vp, gr_w, gr_h As Integer
    
    a_dp = max_p_l - print_depth - l_ht     'Store the available depth left on the page.
   
    'Load the graphic.
    pic.Picture = LoadPicture(g_path)    'Load the graphic into the Picture box and
    s_w = pic.Width                      'store the width and
    s_h = pic.Height                     'height.
 
    Select Case gw  'Set the graphic WIDTH.
        Case 0          'Keep source width.
            gr_w = s_w  'set the width to that of the source.
        Case Is < 1             'This is a fraction, so
            gr_w = l_width * gw 'set the graphic width in terms of the line width.
        Case Else       'Otherwise,
            gr_w = gw   'set the width to the parameter.
    End Select
    If gr_w > l_width Then  'there's no room for the graphic, so
        MsgBox "The original graphic is too wide for the page.", vbOKOnly + vbCritical, "GRAPHIC IMPORT ERROR"
        canceled = True
        Exit Sub
    End If
        
    Select Case gh  'Set the graphic HEIGHT.
        Case 0                      'This is the default, so
            gr_h = gr_w * s_h / s_w 'set the graphic height in proportion to the source size.
        Case Is < 1             'This is a fraction, so
            gr_h = max_p_l * gh 'set the graphic height in terms of the page length.
        Case Else       'Otherwise,
            gr_h = gh   'set the height to the parameter.
    End Select
    If gr_h > a_dp Then 'there's no room for the graphic, so
        MsgBox "The selected height is too tall for the page.", vbOKOnly + vbCritical, "GRAPHIC IMPORT ERROR"
        canceled = True
        Exit Sub
    End If
  
    Select Case hh  'Set the LEFT position.
        Case -1                                 'This is the default, so
            hp = l_marg + (l_width - gr_w) / 2  'centre the graphic.
        Case Else           'Otherwise,
            hp = l_marg + hh 'set from the left margin.
    End Select
        
    Select Case vv  'Set the TOP position.
        Case 0                      'This is the default, so
            vp = print_depth + l_ht  'set the graphic a line space down from the current print depth.
        Case Else               'Otherwise,
            vp = vv + t_marg    'set from the top margin.
    End Select
        
    'DRAW the graphic
    If pp_flg = 0 Then
        Printer.PaintPicture pic.Image, hp, vp, gr_w, gr_h 'Add the picture to the PRINT page,
    Else
        f_pic.PaintPicture pic.Image, hp * ratio, vp * ratio * p_zoom, _
        gr_w * ratio * p_zoom, gr_h * ratio * p_zoom     'Add the picture to the PREVIEW page.
    End If
    
    hh = hp + gr_w                  'Return the horizontal position via the 'hh' parameter.
    
    print_depth = vp + gr_h + l_ht  'Reset the print depth with a line space and
    vv = print_depth                'return the value via the 'vv' parameter.
    
End Sub

'Routine to set text formating.
'Procedure name:    set_font(txx)
'Parameter:         txx;    a text string of formating characters.
'                   sk;     the section concerned, (for backspace calculation).

Private Sub set_font(txx, sk)

    Dim t_l, j, pt, hh As Integer
    Dim p_char, char, n_t, p_fn, b_sp As String
    
    back = 0            'Zero the back space value.
    
    t_l = Len(txx)      'Store the text length and
    
    If t_l = 0 Then 'there are no codes for this block, so
        re_set_font     'reset the font parameters and
        ss_set = False  'clear the Super/Sub flag.
    Else
        For j = 1 To t_l
            p_char = UCase(Mid(txx, j, 1))
            
            'Analyse the letter codes.
            Select Case p_char
                Case "K"    'Backspace, (this is first as it needs the previous format intact).
                    If sk > 0 Then  'Get the size of a back space from the PREVIOUS character.
                        back = Printer.TextWidth(Right(Trim(com(1, sk - 1)), 1))
                    Else                    'or
                        back = 2 * spc_sz   'use a double space.
                    End If
                Case "B"    'Set Bold.
                    Printer.font.Bold = True        'Set the printer font.
                    f_pic.font.Bold = True   'Set the Picture Box font.
                Case "I"    'Set Italic.
                    Printer.font.Italic = True      'Set the printer font.
                    f_pic.font.Italic = True 'Set the Picture Box font.
                Case "U"    'Set Underline.
                    Printer.font.Underline = True   'Set the printer font.
                    f_pic.font.Underline = True   'Set the Picture Box font.
                Case "T"    'Set StrikeThrough.
                    Printer.font.Strikethrough = True   'Set the printer font.
                    f_pic.font.Strikethrough = True   'Set the Picture Box font.
                Case "Y"    'Change to "Symbol" font.
                    Printer.font.Name = sym_font    'Set the fonts to the
                    f_pic.font.Name = sym_font      'stored 'Symbol' font.
                Case "W"    'Change to "WingDings" font.
                    Printer.font.Name = wd_font     'Set the fonts to the
                    f_pic.font.Name = wd_font       'stored 'WingDings' font.
                Case "D"    'Change back to default font.
                    Printer.font.Name = f_name  'Reset the fonts to the
                    f_pic.font.Name = f_name    'original font.
                Case "S"    'Set Superscript.
                    If Not ss_set Then 'No S/P flag is set, so
                        Printer.font.Size = Printer.font.Size * 0.65 '2
                        f_pic.font.Size = f_pic.font.Size * 0.65 '2
                        prntd = print_depth 'Store the current printing depth.
                        print_depth = print_depth - l_ht * 0.3  'set the position and
                        ss_set = True   'flag that Super or Subscript is set.
                    Else                                    'Otherwise,
                        print_depth = prntd - l_ht * 0.3    'set the position from the stored depth.
                    End If
                Case "P"    'Set Subscript.
                    If Not ss_set Then 'No S/P flag is set, so
                        Printer.font.Size = Printer.font.Size * 0.65 '2
                        f_pic.font.Size = f_pic.font.Size * 0.65 '2
                        prntd = print_depth 'Store the current printing depth.
                        print_depth = print_depth + l_ht * 0.3  'set the position and
                        ss_set = True   'flag that Super or Subscript is set.
                    Else                                    'Otherwise,
                        print_depth = prntd + l_ht * 0.3    'set the position from the stored depth.
                    End If
                Case "N"   'Return to normal.
                    re_set_font
                    ss_set = False
                Case Else
                    'If this is a numeric it's part of a font SIZE.
                    If IsNumeric(p_char) Then
                        pt = j + 1                              'Check the next characters
                        n_t = p_char                            'to get
                        Do Until Not IsNumeric(Mid(txx, pt, 1)) 'all of the digits.
                            n_t = n_t + Mid(txx, pt, 1)
                            pt = pt + 1
                        Loop

                        Printer.font.Size = Val(n_t)    'Set the font sizes.
                        f_pic.font.Size = Val(n_t) * ratio * p_zoom
                        j = pt - 1            'Reset the loop variable.
                    Else
                        MsgBox "Code '" + p_char + "' not recognised.", vbOKOnly, "FORMAT CODE ERROR"
                        canceled = True
                        Exit Sub
                    End If
            End Select
        Next
    End If
    
    l_ht = Printer.TextHeight("I")  'Reset the height of a character, (line width), and
    spc_sz = Printer.TextWidth(" ") 'the size of a space in the current font size.
        
End Sub

'Routine to store and replace the current font formating.
'Procedure name:    store_font(s_r)
'Parameter:         s_r;    operation flag, 0 = store, 1 = replace font data.

Sub store_font(s_r)

    If s_r = 0 Then 'store the font data, (only Printer font needed).
        s_f_n = Printer.font.Name
        s_f_z = Printer.font.Size
        s_f_b = Printer.font.Bold
        s_f_i = Printer.font.Italic
        s_f_u = Printer.font.Underline
        s_f_s = Printer.font.Strikethrough
    ElseIf s_r = 1 Then 'replace the font data.
        Printer.font.Name = s_f_n
        Printer.font.Size = s_f_z
        Printer.font.Bold = s_f_b
        Printer.font.Italic = s_f_i
        Printer.font.Underline = s_f_u
        Printer.font.Strikethrough = s_f_s
        
        f_pic.font.Name = s_f_n
        f_pic.font.Size = s_f_z * ratio * p_zoom
        f_pic.font.Bold = s_f_b
        f_pic.font.Italic = s_f_i
        f_pic.font.Underline = s_f_u
        f_pic.font.Strikethrough = s_f_s
    End If
    
End Sub

'Routine to reset text formating to the defaults.
'Procedure name:    re_set_font()

Private Sub re_set_font()

    pp_font             'Reset the font parameters to the stored values and
    If ss_set Then
        print_depth = prntd 'the print depth.
    End If
    
End Sub

'Routine to add a header to a new page.
'Procedure name:    header(tx, f_nm, f_sz, f_mb, Optional aln = 1)
'Parameter:         tx;     header text,
'                   f_nm;   the font name,
'                   f_sz;   the font size,
'                   f_mb;   font Bold flag,
'                   aln;    the alignment, (defaults to 1 = CENTRE).

Public Sub header(tx, f_nm, f_sz, f_mb, Optional aln = 1)
 
    Dim h_m As Integer
    Dim p_fsz As Single
    Dim p_fn As String
    Dim p_fb As Boolean
    
    p_fn = f_name: p_fsz = f_size: p_fb = f_bold    'Store the existing values.
    font (f_nm), (f_sz), (f_mb)                     'Set the font parameters for the header.
 
    h_m = h_marg    'Store the header margin to preserve the value.
    
    If aln < 0 Or aln > 2 Then
        MsgBox "Alignment codes must be 0, 1 or 2", vbOKOnly + vbCritical, "'add_text' ERROR"
        canceled = True
    Else
        add_text tx, 0, True, h_m, 0, aln   'Align the heading text.
        print_depth = t_marg                'Set initial print position for the next page to the top margin.
    
        font (p_fn), (p_fsz), (p_fb)    'Restore the original font parameters.
    End If
    
End Sub

'Routine to add a footer to a completed page,
'date at left using 'super_date', page number central.
'Procedure name:    footer(pnf, f_nm, f_sz, f_mb, Optional al1 = 0, Optional al2 = 1)
'Parameter:         pnf;    display the ABSOLUTE page number, False = no number,
'                   f_nm;   the font name,
'                   f_sz;   the font size,
'                   f_mb;   font Bold flag,
'                   al1;    date alignment, (defaults to 0 = LEFT),
'                   al2;    page number alignment, (defaults to 1 = CENTRE).

Public Sub footer(pnf, f_nm, f_sz, f_mb, Optional al1 = 0, Optional al2 = 1)

    Dim pd, ppd As Integer, p_fn As String, p_fsz As Single, p_fb As Boolean
    Dim dte As String, tx As String

    If al1 < 0 Or al1 > 2 Or al2 < 0 Or al2 > 2 Then
        MsgBox "Alignment codes must be 0, 1 or 2", vbOKOnly + vbCritical, "'add_text' ERROR"
        canceled = True
    Else
        If pnf And al1 = al2 Then  'both alignments coincide!
            MsgBox "Both alignments are the same!", vbOKOnly + vbCritical, "'footer' ERROR"
            canceled = True
        Else
            p_fn = f_name: p_fsz = f_size: p_fb = f_bold    'Store the existing values.
            font (f_nm), (f_sz), (f_mb)         'Set the font parameters for the header,
    
            dte = super_date(Now, True)  'Format the date with superscript.
        
            If pnf Then
                tx = "Page " + CStr(p_a_num)    'Create the Page number text.
            Else
                tx = ""
            End If

            pd = (max_p_l + b_marg - f_marg)    'Set the printing depth,
            ppd = pd                            'store it separately and
            add_f_text dte, 0, False, ppd, 0, al1 'add the date.
        
            If tx <> "" Then
                add_text tx, 0, True, pd, 0, al2 'Display the aligned page number.
            End If
    
            font (p_fn), (p_fsz), (p_fb)   'Restore the original font parameters.
        End If 'error.
    End If
    
 End Sub

'Routine to find the start position to centre a piece of text.
'The Print/Preview flag is ignored as the position is found in terms of
'PRINTING text which is scaled later.

'Function name:     centre(tx)
'Parameter:         tx;    the text,

Public Function centre(tx)

    Dim l_txt As Integer

    l_txt = Printer.TextWidth(tx)  'Get the text width.
 
    centre = (l_width - l_txt) / 2    'Return the X-position.
 
End Function

'Routine to create a 'super' date string with superscript 'st', 'nd', 'rd' and 'th'.
'Function name: super_date(dt, f)
'Parameter:     dt; the date.
'               f;  the formated text flag, (True = add custom formating flags).
'                   N.B. these will only work if 'add_f_text' is used.

Public Function super_date(dt As Date, f)

    Dim str1, str2, str3, super As String
    Dim d_n, f_d As Integer
    
    str1 = Format(dt, "mmmm")   'Store the month.
    str2 = Format(dt, "d")      'Get the day number,
    f_d = Val(str2) \ 10        'its first digit and
    d_n = Val(str2) Mod 10      'convert it to 0 - 9.
    str3 = Format(dt, "yyyy")   'Store the year.
 
    Select Case d_n             'Use 'd_n' to select the superscript text.
        Case 0, 4, 5, 6, 7, 8, 9
            If f Then
                super = "\s/th\n/"
            Else
                super = " th"
            End If
        Case 1
            If f Then
                super = "\s/st\n/"
            Else
                super = " st"
            End If
        Case 2
            If f Then
                super = "\s/nd\n/"
            Else
                super = " nd"
            End If
        Case 3
            If f Then
                super = "\s/rd\n/"
            Else
                super = " rd"
            End If
    End Select
    
    If f_d = 1 Then             'Over-ride 'st', 'nd' & 'rd'
        If f Then               'if it's 11th, 12th or 13th.
            super = "\s/th\n/"
        Else
            super = " th"
        End If
    End If
    
    super_date = str1 + " " + str2 + super + ", " + str3   'Return the date string.
    
End Function

'Routine to check the current alignment of text printing.
'This isn't used by the Class but is useful to have around.
'[VB defaults are LEFT and TOP.

'Procedure name:    check_align(obj)
'Parameter:         obj;    the qualifying object being checked, Form, Picture Box, Printer, etc.

Private Sub check_align(obj As Object)

    Dim refpoint As Long
    
    refpoint = GetTextAlign(obj.hDC) 'Get the text alignment setting of the object.
 
    If (refpoint And TA_CENTER) = TA_CENTER Then
        MsgBox "Text will be displayed centered horizontally."
    ElseIf (refpoint And TA_RIGHT) = TA_RIGHT Then
        MsgBox "Text will be displayed right-justified."
    Else  'Assume ref. point on left edge of block.
        MsgBox "Text will be displayed left-justified."
    End If
    
    If (refpoint And TA_BASELINE) = TA_BASELINE Then
        MsgBox "Text will be lined to the BASE of the letters."
    ElseIf (refpoint And TA_BOTTOM) = TA_BOTTOM Then
        MsgBox "Text will be lined to the BOTTOM of the bounding box."
    Else
        MsgBox "Text will be lined to the TOP of the bounding box."
    End If
    
End Sub

