VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PPpages"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'COLLECTION CLASS MODULE TO PROVIDE A COLLECTION
'OF PAGES OF PP_PAGE OBJECTS.

'There should be nothing here that is specific to any
'particular  project except that:-

'1 The error routines require a Public Boolean, 'canceled' in the main program.

'2 The routines use a main program procedure, 'create_page', to write/re-write
'  specific page data for PRINTING or PREVIEW.
'  (The alternative would be to have a specific 'page' object as a method. However, it was
'   felt that coding the necessary degree of flexibilty of font and format would be too
'   cumbersome).

'3 The FlatScrollBar controls require that the Microsoft Windows Common Controls - 2
'  6.0(SP4), MSCOMCT2.OCX, be included in the project components.

'4 The following icon files are required in the application folder:-
'       mag-plus.ico
'       mag-minus.ico

'The code is written for units of inches but this can be changed by changing the value
'of the 'units' variable from 0 to 1. This could be provided as a user option.

' Philip Bolt, 2005, 2006, 2007
'Last updated: 8/3/7

Option Explicit
    
'The Collection defined below is Private and used only to enumerate the pages.
'If its declaration is made Public, it will be available in the main program.

'The collection must hold the range of pages to be printed. If there is an option of
'ODD or EVEN page printing, this must be reflected in the numbering of the pages stored.
'e.g. a range of 3 odd pages starting at P.7 must be stored as 7, 9, 11 and 'p_inc'
'set to 2 so that the correct pages will be selected when the Scroll Bar is moved.

Private PP_pgs As New Collection    'PPpages' collection.

Private Const n_ctrl As Integer = 6     'Number of controls.
Private Const n_parm As Integer = 12    'Number of size parameters.
Private Const p_con_i As Single = 1440  'Set the conversion factor for twips/inch.
Private Const p_con_m As Single = 56.69 'Set the conversion factor for twips/mm.

'Zoom parameters.
Private Const z_f_v As Single = 0.2 'Zoom factor.
Private Const z_max As Integer = 10 'Maximum value of zoom count.
Private z_cnt As Integer            'Zoom increment count.
Private z_inc As Integer            'Zoom direction multiplier.
Private zoom_on As Boolean          'Zoom on flag.

'Units parameters.
Private units() As Variant          'Data to set the units to inches, (0), or mm, (1).
Private unit As Integer             'Units = inches' flag, 1 gives mm.

'Display parameters.
Private ht As Integer               'Screen height.
Private wd As Integer               'Screen width.
Private pic_ht As Integer           'Preview picture height.
Private pic_wd As Integer           'Preview picture width.
Private bar_width As Integer        'Scroll Bar width.

'Selected page data.
Private n_pges As Integer           'Number of pages.
Private p1 As Integer               'The first page number.
Private p_inc As Integer            'The page increment, 2 for ODD or EVEN pages.

'Current page data.
Private p_n As Integer              'Current page number.
Private flg As Integer              'Current Print/Preview flag.
Private z_m As Single               'Current zoom factor.

'Scroll Bar parameters.
Private p_rnge As Integer           'Range of scroll Bar values per page.
Private pvp As Integer              'Previous vertical Bar value.
Private v_dif As Integer            'Vertical Bar difference value.

'Paper parameters
Private parm() As Single            'Parameter storage array.
Private PgHeight As Single          'Printing page height.
Private PgWidth As Single           'Printing page width.
Private t_marg As Single            'Top margin.
Private h_marg As Single            'Header margin.
Private b_marg As Single            'Bottom margin.
Private f_marg As Single            'Footer margin.
Private l_marg As Single            'Left margin.
Private r_marg As Single            'Right margin.
Private LRgap As Single             'Printer dead spaces.
Private TBgap As Single
Private max_p_l As Single           'Maximum page length.
Private l_width As Single           'Line width.
Private p_size As Integer           'Page size, vbPRPSA4 = 9, vbPRPSA5 = 11.
Private p_orient As Integer         'Page orientation, vbPRORPortrait = 1, vbPRORLandscape = 2.
Private ratio As Single             'Page scaling factor.
Private p_rat As Single             'Ratio of paper ht/wd.
Private p_con As Single             'mm/inches to twips conversion factor.
Private p_col As Long               'Paper' colour.

'FONT parameters
Private f_parm() As Variant         'Font name, (0), Size, (1), Bold, (2), Italic, (3),
'                                    Underline, (4), Strikethrough, (5), Colour (6),
'                                    Weight, (7)

'Array for the return of the margins and paper width & height.
Private p_data(6) As Single         '0 = LEFT MARGIN, 1 = RIGHT MARGIN, 2 = TOP MARGIN,
                                    '3 = BOTTOM MARGIN, 4 = PAPER WIDTH, 5 = PAPER HEIGHT.
                                    '6 = mm/inches flag, (1 or 0).
                                    
Private path_way As String          'Application path string.

Private in_use As Boolean           'Scroll Bar in use flag.
Private setup As Boolean            'Data set-up flag.
Private printer_selected As Boolean 'Printer has been selected.
Private no_v_bar As Boolean         'Vertical scroll bar hidden flag.

Private frm As Object               'The PREVIEW form.
Private ctrl() As Object            'PREVIEW form controls.
Private cdc As Object               'Common Dialog Control replacement object.

'Object constructor.
'Run when the object is created.

Private Sub Class_Initialize()

    If Not setup Then
        set_up
        setup = True
    End If
 
End Sub

'Routine to add a new page to the pages collection.
'Function name: Add(ByVal pge As integer)
'Parameter:     l_pge;   the LOCAL page number.
'               a_pge;   the ABSOLUTE page number.

Public Function Add(ByVal l_pge As Integer, Optional a_pge As Integer) As PPpage

    Dim j As Integer
    Dim ppp As New PPpage, PPp_f As PPpage      'Dimension the 'Page' objects.
 
    ppp.page_numbers l_pge, a_pge               'Store the page numbers.
 
    PP_pgs.Add ppp, CStr(ppp.page_number(0))    'Add the page to the collection with
                                                'the LOCAL page number as ID.
    For j = 0 To 12
        ppp.set_sizes j, parm(j)    'Transfer the default page structure values,
    Next

    ppp.font_defaults f_parm()      'Transfer the font defaults to the page.
    
    n_pges = PP_pgs.Count           'Store the number of pages.
    
    Set PPp_f = PP_pgs.Item(1)      'Select the first page in the set and
    p1 = PPp_f.page_number(0)       'store the first page number.
    
    set_v_bar 0                 'Initialise the vertical Scroll Bar.
    
    Set Add = ppp               'Return the object.
 
End Function

'Routine to delete a page object from the collection.
'Procedure name:    Delete(ByVal pge As integer)
'Parameter:         pge;   the LOCAL page number.

Public Sub Delete(ByVal pge As Integer)

    Dim ppp As PPpage   'Dimension a 'Page' object and
    Dim j As Integer    'the loop variable.
    
    If n_pges > 0 Then      'There are some pages in the collection, so
        For j = 1 To n_pges 'check each one.
            If PP_pgs(j).page_number(0) = pge Then  'this is the one, so
                Set ppp = PP_pgs.Item(j)    'set the object variable to the page,
                PP_pgs.Remove j             'remove it from the collection and
                Set ppp = Nothing           'release the handle.
            End If
        Next
        n_pges = PP_pgs.Count   'Re-store the number of pages.
    End If
    
End Sub

'Routine to close down a PP_pages object.
'Procedure name:    Class_Terminate()

Private Sub Class_Terminate()

    Dim ppp As PPpage     'Dimension a 'Page' object.
    
    For Each ppp In PP_pgs  'Process each page in the collection,
        Set ppp = Nothing   'close it.
    Next

    setup = False           'Flag that data is NOT set-up and
    
    Unload frm

End Sub

'Routine to set up an enumerator for the collection.
'NewEnum must return the IUnknown interface of a
'collection's enumerator.

'This function is hidden, (Select Tools, Procedure Attributes, Select 'NewEnum'
'                          Advanced, Check 'Hide this member', set 'Procedure ID' to -4)

Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"

   Set NewEnum = PP_pgs.[_NewEnum]
   
End Function

'Routine to initialise the Pages data.
'Procedure name:    set_up

Private Sub set_up()
 
    Dim fn As String
    
    path_way = App.path + "\"   'Define the application path string.
    
    Set frm = preview
    
    Set cdc = New CDC_class     'Define the CDC object.
    
    unit = 0                    'Units flag, 0 = inches, 1 = mm.
    
    ReDim units(3, 1)
    units(0, 0) = p_con_i       'The zeroth values are the twips conversion factors.
    units(0, 1) = p_con_m
    units(1, 0) = 5             'The 1st. values are the ScaleModes,
    units(1, 1) = 6             '( 5 = inches, 6 = mm.).
    units(2, 0) = cdlPSDInThousandthsofInches  'The 2nd. values are Page Setup flags.
    units(2, 1) = cdlPSDInHundredthsofMillimeters
    
    ReDim ctrl(n_ctrl)                      'Dimension the control array.
    Set ctrl(0) = preview.Picture1          'Preview 'Window' picture box
    Set ctrl(1) = preview.Picture2          'Preview 'Child' picture Box.
    Set ctrl(2) = preview.FlatScrollBar1    'Preview VERTICAL scroll bar
    Set ctrl(3) = preview.FlatScrollBar2    'Preview HORIZONTAL scroll bar.
    Set ctrl(5) = preview.Image1(4)         'Mouse + magnifying glass image.
    Set ctrl(6) = preview.Image1(5)         'Mouse - magnifying glass image.
    
    ht = Screen.Height      'Store the screen height and
    wd = Screen.Width       'width.
   
    bar_width = 340         'Scroll Bar widths.
    
    On Error GoTo etrap     'Set error trap.
    
    ctrl(5).Picture = LoadPicture(path_way + "mag-plus.ico")   'Load the Zoom mouse icons.
    ctrl(6).Picture = LoadPicture(path_way + "mag-minus.ico")
    
    On Error GoTo 0     'Clear the 'On error' jump.
    
    ReDim f_parm(7) 'FONT parameters. These can be changed by 'set_font'
    f_parm(0) = "Arial"     'Set defaults for the font NAME,
    f_parm(1) = 12          'and SIZE with
    f_parm(2) = False       'no BOLD,
    f_parm(3) = False       'no ITALIC,
    f_parm(4) = False       'no UNDERLINE and
    f_parm(5) = False       'no STRIKETHROUGH.
    f_parm(6) = 0           'Text Colour, (Forecolor) = black
    f_parm(7) = 400         'Weight = Normal.
    
    ReDim parm(n_parm)      'Default margins, etc.
    
    p_data(0) = 0.75 * units(0, 0) / units(0, unit) 'LEFT margin,
    p_data(1) = 0.75 * units(0, 0) / units(0, unit) 'RIGHT,
    p_data(2) = 0.75 * units(0, 0) / units(0, unit) 'TOP,
    p_data(3) = 0.75 * units(0, 0) / units(0, unit) 'BOTTOM,
    p_data(4) = 0                                   'Paper L x B,
    p_data(5) = 0                                   '(not used but available.)
    p_data(6) = unit    'Flag for measures in "s

    p_con = units(0, unit)      'Define the conversion factor.
    
    p_col = RGB(255, 255, 255)  'Define the default paper colour and
    parm(11) = p_col            'store it as a parm.
    
    check_paper_size            'Set the default paper size.
    
    z_cnt = 0                   'Initialise the zoom stages count and
    z_inc = 1                   'the multiplier value.
    
    zoom_on = False             'Clear the zoom,
    printer_selected = False    'Printer Selected and
    no_v_bar = False            'hidden V-bar flags.
    
    Exit Sub
    
etrap: 'ERROR TRAP
    error_report Err.Number, fn, "'set_up'"
    
End Sub

'Routine to remove a set of pages from the Pages collection.
'Procedure name:    mt

Public Sub mt()

    Dim p_cnt, j, lst As Integer, ppp As PPpage 'Dimension the variables.
    
    p_cnt = PP_pgs.Count    'Count the pages.
    
    If p_cnt > 0 Then 'there are pages, so
        For j = p_cnt To 1 Step -1  'process until all pages removed.
            Set ppp = PP_pgs(j)     'Set the 'PP_page' variable to the page,
            PP_pgs.Remove j         'remove it from the collection and
            Set ppp = Nothing       'delete it.
        Next
        
        printer_selected = False    'Clear the Printer Selected flag.
    End If 'there are pages.
    
End Sub

'Routine to define the page properties of a new PPpages object.
'Property Set name: page_parms(ByVal p_sz As variant, ByVal p_or As variant,
'                              ByVal t_m As variant, ByVal h_m As variant,
'                              ByVal b_m As variant, ByVal f_m As variant,
'                              ByVal l_m As variant, ByVal r_m As variant,
'                              ByVal p_i As variant, ByVal p_cl as variant)

'Parameters:    p_sz;   the PAPER SIZE, vbPRPSA4 = 9, vbPRPSA5 = 11,
'               p_or;   the PAGE ORIENTATION, vbPRORPortrait = 1, vbPRORLandscape = 2,
'               t_m;    the TOP margin,
'               h_m;    the HEADER margin,
'               b_m;    the BOTTOM margin,
'               f_m;    the FOOTER margin,
'               l_m;    the LEFT margin,
'               r_m;    the RIGHT margin,
'               p_i;    the page increment, 1 for ALL, 2 for ODD or EVEN.
'               p_cl;   the paper colour.

Public Sub page_parms(Optional ByVal p_sz As Variant, Optional ByVal p_or As Variant, _
                      Optional ByVal t_m As Variant, Optional ByVal h_m As Variant, _
                      Optional ByVal b_m As Variant, Optional ByVal f_m As Variant, _
                      Optional ByVal l_m As Variant, Optional ByVal r_m As Variant, _
                      Optional ByVal p_i As Variant, Optional ByVal p_cl As Variant)

    If Not IsMissing(p_sz) Then
        p_size = CInt(p_sz)         'PAGE SIZE,
    End If

    If Not IsMissing(p_or) Then
        p_orient = CInt(p_or)       'PAGE ORIENTATION,
    End If

    If Not IsMissing(t_m) Then
        t_marg = CSng(t_m)          'TOP MARGIN,
    End If

    If Not IsMissing(h_m) Then
        h_marg = CSng(h_m)          'HEADER MARGIN,
    End If
    
    If Not IsMissing(b_m) Then
        b_marg = CSng(b_m)          'BOTTOM MARGIN,
    End If

    If Not IsMissing(f_m) Then
        f_marg = CSng(f_m)          'FOOTER MARGIN,
    End If

    If Not IsMissing(l_m) Then
        l_marg = CSng(l_m)          'LEFT MARGIN,
    End If

    If Not IsMissing(r_m) Then
        r_marg = CSng(r_m)          'RIGHT MARGIN,
    End If

    If Not IsMissing(p_i) Then
        p_inc = CInt(p_i)           'PAGE NUMBER INCREMENT,
    End If

    If Not IsMissing(p_cl) Then
        p_col = CLng(p_cl)          'PAPER COLOUR.
    End If

    parm(2) = t_marg: parm(3) = h_marg: parm(4) = b_marg 'Store the values in the
    parm(5) = f_marg: parm(6) = l_marg: parm(7) = r_marg 'parameter array.
    parm(8) = p_inc: parm(11) = p_col
    
End Sub

'Set of property routines to allow access to page structure data.
'The item required is selected by the 'opt' parameter.
'Property name:  page_data(opt)

Public Property Get page_data(opt As Integer)

    Select Case opt
        Case 0
            page_data = max_p_l 'Maximum page length
        Case 1
            page_data = l_width 'Line width
        Case 2
            page_data = t_marg  'top margin.
        Case 3
            page_data = h_marg  'header margin,
        Case 4
            page_data = b_marg  'bottom margin.
        Case 5
            page_data = f_marg  'footer margin.
        Case 6
            page_data = l_marg  'left margin.
        Case 7
            page_data = r_marg  'right margin.
        Case 8
            page_data = ratio   'the Print/Preview ratio.
        Case 9
            page_data = LRgap   'the Printer Width dead spaces.
        Case 10
            page_data = TBgap
        Case 11
            page_data = z_m     'the current Zoom factor.
        Case 12
            page_data = p_col   'the 'paper' colour.
        Case Else
            MsgBox "Options must be in the range 0 to 12", vbOKOnly + vbCritical, "'PPpages:page_data' ERROR"
            canceled = True
    End Select
 
End Property

'Routine to read the number of pages in a collection.
'Property name: num_pge

Public Function num_pge()

    num_pge = n_pges
    
End Function

'Routine to set the PAGE INCREMENT for a range of pages.
'Property name: set_pinc(pnc As integer)
'Parameter:     pnx;    the increment.

Public Sub set_pinc(pnc As Integer)
 
    If pnc > 0 And pnc < 3 Then
        p_inc = pnc 'Store the increment value.
    Else
        MsgBox "The value must be 1 or 2.", vbOKOnly + vbCritical, "'set_pinc' ERROR"
    End If
 
End Sub

'Routine to set up the PRINT PREVIEW form and Picture Boxes, etc.
'Procedure name:    set_preview_form

Private Sub set_preview_form()

    With frm
        .ScaleMode = 1
        .Height = pic_ht + bar_width
        .Width = pic_wd + bar_width
        .Top = 200
        .Left = (wd - pic_wd) / 2
    End With
 
     With ctrl(0)   'Picture1
        .ScaleMode = 1
        .Height = pic_ht
        .Width = .Height / p_rat
        .Top = bar_width
        .Left = 0
    End With

    With ctrl(1)    'Picture2
        .ScaleMode = 1
        .Height = ctrl(0).Height
        .Width = ctrl(0).Width
        .Top = 0
        .Left = 0
        .MousePointer = 99              'Set for a custom pointer and
        .MouseIcon = ctrl(5).Picture    'load the 'Mag+' icon.
    End With
        
    With ctrl(2)                   'This is the VERTICAL scroll bar.
        .Height = ctrl(0).Height
        .Width = bar_width
        .Top = ctrl(0).Top
        .Left = ctrl(0).Left + ctrl(0).Width
        '.Visible = True
    End With
    
    With ctrl(3)                   'This is the HORIZONTAL scroll bar.
        .Height = bar_width
        .Width = ctrl(0).Width
        .Top = ctrl(0).Height + bar_width
        .Left = ctrl(0).Left
        .Visible = False            'Hide the horizontal bar initially.
    End With
 
    frm.ScaleMode = units(1, unit)      'Define the ScaleModes.
    ctrl(0).ScaleMode = units(1, unit)
    ctrl(1).ScaleMode = units(1, unit)

End Sub

'Routine to check the selected paper size, generate a printer scale factor and then
'set up the PRINT PREVIEW form. All sizes are defined in PRINTER terms.
'Procedure name:    check_paper_size()

Private Sub check_paper_size()

    Dim psm, fsm, sm As Integer
    Dim HeightRatio, WidthRatio As Single

    p_orient = Printer.Orientation  'Store the page orientation and
    p_size = Printer.PaperSize      'the paper size.
    
    set_preview_pic
    
    Printer.ScaleMode = units(1, unit) 'Set printer units.
    PgWidth = Printer.Width / p_con
    PgHeight = Printer.Height / p_con

    LRgap = (PgWidth - Printer.ScaleWidth) / 2
    TBgap = (PgHeight - Printer.ScaleHeight) / 2
    
    sm = ctrl(1).ScaleMode
    ctrl(1).ScaleMode = units(1, unit) 'Set Picture Box units.
    ' Compare the height and width ratios to determine the
    ' Ratio to use and how to size the picture box:
    HeightRatio = ctrl(1).ScaleHeight / PgHeight
    WidthRatio = ctrl(1).ScaleWidth / PgWidth

    If HeightRatio < WidthRatio Then
        ratio = HeightRatio
    Else
        ratio = WidthRatio
    End If

    ' Set default properties of picture box to match printer
    ' There are many that you could add here:
    ctrl(1).Scale (0, 0)-(PgWidth, PgHeight)
    ctrl(1).FontName = Printer.FontName
    ctrl(1).FontSize = Printer.FontSize * ratio
    ctrl(1).Cls

    l_marg = p_data(0)              'Left margin, in units.
    r_marg = p_data(1)              'Right margin,
    t_marg = p_data(2)              'Top margin,
    b_marg = p_data(3)              'Bottom margin.
 
    h_marg = t_marg / 2             'Header margin, (half of Top),
    f_marg = b_marg / 2             'Footer margin, (half of Bottom).

    max_p_l = PgHeight - t_marg - b_marg    'Maximum page length in units.
    l_width = PgWidth - r_marg - l_marg     'Text line width in units.
 
    parm(0) = max_p_l: parm(1) = l_width    'Store them in the parameter array,
    parm(2) = t_marg: parm(3) = h_marg
    parm(4) = b_marg: parm(5) = f_marg
    parm(6) = l_marg: parm(7) = r_marg
    parm(8) = max_p_l: parm(9) = LRgap      'with the dead space.
    parm(10) = TBgap: parm(12) = ratio

End Sub

'Routine to set the size of the Print Preview page.
'The Picture box height is a constant, 'pic_ht'.
'Procedure name:    set_preview_pic

Private Sub set_preview_pic()

    p_rat = Printer.Height / Printer.Width
    
    If p_orient = 1 Then  'this is Portrait, so
        pic_ht = ht - 1400          'Set the picture box height to leave a 200 top margin and
        pic_wd = pic_ht / p_rat     'the width to be in paper size ratio.
    Else
        pic_wd = wd - 1000
        pic_ht = pic_wd * p_rat
    End If
    
    set_preview_form            'Adjust the preview form size.
    
End Sub

'Routine to modify the vertical Scroll Bar
'Procedure name:    change_v_bar(vpp)
'Parameter:         vpp;    the current vertical Bar value.

Public Sub change_v_bar(vpp)
 
    If Not in_use Then
        Dim pn, p_pos, n_pos As Integer
        Dim tp As Single
        
        pvp = vpp                       'Store the value.
    
        check_arrows vpp, 2             'Check the arrows.
    
        pn = (vpp \ p_rnge) * p_inc + p1 'Get the indicated page number.
 
        If pn <> p_n Then  'this isn't the same page, so
            p_n = pn                   'Store the new page number and
            create_page flg, z_m, p_n  'display a new page.
           
            If vpp > pvp Then   'Check the direction of movement and
                tp = 0          'set the start of page
            Else                'position
                tp = -v_dif     'accordingly.
            End If
 
            ctrl(1).Top = tp / units(0, unit)   'Reset the Top position.
        Else
            If zoom_on Then
                p_pos = vpp - (p_n - p1) * p_rnge       'Store the position value in page,
                n_pos = p_pos * v_dif / (p_rnge - 1)    'find the fraction of 'v_dif' concerned and
                ctrl(1).Top = -n_pos / units(0, unit)   'move the Picture2 Top.
            End If
        End If
    End If 'not in use.
 
End Sub

'Routine to modify the horizontal Scroll Bar
'Procedure name:    change_h_bar(hpp)
'Parameter:         hpp;    the current horizontal Bar value.

Public Sub change_h_bar(hpp)
 
    check_arrows hpp, 3  'Check the arrows.
    
    ctrl(1).Left = -hpp / units(0, unit) 'Position the Picture2 box in the window.

End Sub

'Routine to check the scroll bar arrows.
'Procedure name:    check_arrows(pnn, sbn)
'Parameters:        pnn;    the position value, (usually the page number for the V Bar).
'                   sbn;    the Scroll Bar control value, 2=VERTICAL, 3=HORIZONTAL.

Private Sub check_arrows(pnn, sbn)

    Select Case pnn
        Case ctrl(sbn).Min
            ctrl(sbn).Arrows = cc2RightDown
        Case ctrl(sbn).Max
            ctrl(sbn).Arrows = cc2LeftUp
        Case Else
            ctrl(sbn).Arrows = cc2Both
    End Select
    
End Sub

'Routine to switch the zoom direction.
'Procedure name:    switch_zoom

Public Sub switch_zoom()

    z_inc = 0 - z_inc       'Toggle the zoom increment multiplier and
    ctrl(1).MouseIcon = ctrl((11 - z_inc) / 2).Picture    'load the correct 'Mag' icon.
    
End Sub

'Routine to zoom in or out.
'Procedure name:    zoom

Public Sub zoom()
 
    z_cnt = z_cnt + z_inc   'Update the zoom stages count.
    
    If z_cnt < 0 Then       'If the count goes negative,
        z_cnt = 0           'set it to zero,
        ctrl(1).MouseIcon = ctrl(5).Picture    'load the 'Mag+' icon and
        z_inc = 1           'set the multiplier to zoom IN.
    End If
    
    If z_cnt > z_max Then   'If the count goes too high,
        z_cnt = z_max       'set it to the maximum.
        ctrl(1).MouseIcon = ctrl(6).Picture    'load the 'Mag-' icon and
        z_inc = -1          'set the multiplier to zoom OUT.
    End If
    
    z_m = 1 + z_f_v * z_cnt    'Set the zoom factor.
 
    If z_cnt <> 0 Then
        zoom_on = True      'Flag that zoom is on and
        zoom_i_o            'zoom in or out.
    Else                    'Otherwise,
        zoom_on = False     'flag that zoom is off and
        re_set_zoom         'reset the display.
    End If
    
End Sub

'Routine to zoom IN or OUT on the display by a set amount.
'Procedute name:    zoom_i_o()

Private Sub zoom_i_o()

    If no_v_bar Then    'this is a single page and the V-bar is hidden, so
        ctrl(2).Visible = True          'show the vertical bar,
        frm.Width = pic_wd + bar_width  'widen the form and
        no_v_bar = False                'clear the flag.
    End If
    
    ctrl(1).Cls                         'Clear the page and
    With ctrl(1)                        're-size the Picture box
        .Height = ctrl(0).Height * z_m  'in terms of Picture1.
        .Width = ctrl(0).Width * z_m
        .Top = 0
        .Left = 0
    End With
    ctrl(1).Scale (0, 0)-(PgWidth, PgHeight)    'Rescale to fit the wider box.
    
    frm.Height = frm.Height + bar_width 'Increase the height of the form to
                                        'take the horizontal bar.
    set_h_bar                           'Set and display the Scroll Bars.
    set_v_bar (p_n - p1) * p_rnge       'Set the vertical Bar for the page.
    
    create_page 1, z_m, p_n  'Display the new page.

End Sub

'Routine to zoom OUT on the display by a set amount.
'Procedute name:    re_set_zoom

Private Sub re_set_zoom()

    z_m = 1             'Set the zoom factor to unity.
        
    ctrl(1).Cls                     'Clear the page and
    With ctrl(1)                    're-size the Picture box.
        .Height = ctrl(0).Height    'to the original.
        .Width = ctrl(0).Width
        .Top = 0
        .Left = 0
    End With
    ctrl(1).Scale (0, 0)-(PgWidth, PgHeight)    'Rescale to match the changed box.
    
    frm.Height = frm.Height - bar_width 'Reduce the height of the form.
    
    ctrl(3).Visible = False             'Hide horizontal Scroll Bar.
        
    ctrl(1).MouseIcon = ctrl(5).Picture 'Load the 'Mag+' icon.
    
    If n_pges = 1 Then
        hide_v_bar                  'Hide the V-bar for a single page.
    Else                            'Otherwise,
        set_v_bar 10 * (p_n - p1)   're-set it.
    End If
    
    create_page 1, z_m, p_n  'Display the new page.
    
End Sub

'Routine to set up the VERTICAL, scroll box.
'Procedure name:    set_v_bar(cvp)
'Parameter:         cvp;    current vertical position.

Private Sub set_v_bar(cvp)
 
    in_use = True                   'Set the in-use flag.
    p_rnge = Int(32767 / n_pges)    'Set the range of scroll bar values per page.
    
    If Not zoom_on Then     'Set the VERTICAL Scroll Bar for normal display.
        v_dif = 0           'Set value for Picture Box differences and
        p_rnge = 10         'the range of values per page.
            
        With ctrl(2)
            .Max = 10 * (n_pges - 1)
            .Min = 0
            .LargeChange = 10
            .SmallChange = 10
            .Value = cvp
        End With
    Else                    'Otherwise, set it for an enlarged display.
        v_dif = (ctrl(1).Height - ctrl(0).Height) * units(0, unit)   'Store the difference in height.
        With ctrl(2)
            .Max = n_pges * p_rnge - 3   'Approximate max. to 32767.
            .Min = 0
            .LargeChange = Int(.Max / n_pges)   '1 page a section.[was n_pges-1]
            .SmallChange = p_rnge / 100         '1/100th of a page.
            .Value = cvp
        End With
    End If
            
    pvp = cvp   'Set up the intial, stored value.
 
    in_use = False  'Clear the in-use flag.
    
End Sub

'Routine to set up the parameters of the HORIZONTAL Scroll Bar.
'Procedure name:      set_h_bar.

Private Sub set_h_bar()

    in_use = True
        With ctrl(3)
            .Max = (ctrl(1).Width - ctrl(0).Width) * units(0, unit) 'Set the maximum to the difference
            .Min = 0                                                'in current units.
            .SmallChange = Int(.Max / 10)
            .LargeChange = Int(.Max / 5)
            .Value = .Min
            .Visible = True
        End With
    in_use = False

End Sub

'Routine to show the preview form.
'The Line command draws a box around the FORM margins for testing.
'Procedure name:    show_preview

Private Sub show_preview()

    If n_pges = 1 And Not zoom_on Then  'This is a single page, normal display, so
        hide_v_bar                      'hide the V-bar.
    End If

    With frm
        .Picture1.Visible = True    'Display both
        .Picture2.Visible = True    'Picture Boxes,
        .Visible = True             'show the form,
        .Picture2.Line (l_marg, t_marg)-(r_marg + l_width, max_p_l + t_marg), , B
        .Label1.Caption = CStr(100 * z_m) + "%" 'Display the enlargement factor.
        .SetFocus                   'with Focus.
    End With
    
End Sub

'Routine to hide the V-bar for a single page.
'Procedure name:    hide_v_bar

Private Sub hide_v_bar()

    ctrl(2).Visible = False 'Hide the vertical bar,
    frm.Width = pic_wd      'shrink the form and
    no_v_bar = True         'flag that this has been done.
        
End Sub

'Routine to close down the preview form.
'Procedure name:    close_preview

Public Sub close_preview()

    If zoom_on Then     'If zoom is on,
        zoom_on = False 'clear the flag.
    End If
    
    frm.Visible = False 'Hide the PREVIEW form.
    
End Sub

'Routine to PRINT or PREVIEW a page.
'The Line command draws a box around the PRINTER margins for testing.
'Procedure name:    print_it

Public Sub print_it(pn)
 
    Dim ppp As PPpage, indx As Integer
 
    p_n = pn    'Store the page number.
 
    indx = find_index((pn))     'Find the collection index for this page.
    
    If indx > 0 Then
        Set ppp = PP_pgs.Item(indx) 'select the page in the collection,
        flg = ppp.page_flag         'read the Print/Preview flag and
        z_m = ppp.page_data(11)     'the zoom factor.

        If flg = 0 Then
            Printer.Line (l_marg - LRgap, t_marg - TBgap)-(r_marg + l_width - LRgap, _
                          max_p_l + t_marg - TBgap), , B
            Printer.EndDoc  'Print the page.
        Else
            show_preview    'Show the Preview form.
        End If
    End If 'index Ok, (Errors dealt with by 'find_index').

End Sub

'Routine to find the page for a specific LOCAL page number.
'Property name: find_page(pn)
'Parameter:     pn; the LOCAL page number.

Public Property Get find_page(pn As Integer)

    Dim p_cnt As Integer, j As Integer, pnn As Integer, indx As Integer
    
    indx = 0    'Initialise the index value.
    
    p_cnt = PP_pgs.Count                'Count the pages.
    If p_cnt > 0 Then
        For j = 1 To p_cnt                  'Check each page,
            pnn = PP_pgs(j).page_number(0)  'storing the LOCAL page number.
            If pnn = pn Then                'Compare this with the target number.
                indx = j                    'Store the index, if found, and
                j = p_cnt                   'terminate the loop.
            End If
        Next
    End If
    
    If indx = 0 Then 'there is an error.
        MsgBox "Page " + CStr(pn) + " isn't indexed!", vbOKOnly + vbCritical, "'find_page' ERROR"
        canceled = True
        Set find_page = Nothing
    Else
        Set find_page = PP_pgs(indx)
    End If
     
End Property

'Routine to find the collection index number for a specific LOCAL page number.
'Property name: find_index(pn)
'Parameter:     pn; the LOCAL page number.

Public Property Get find_index(pn As Integer)

    Dim p_cnt As Integer, j As Integer, pnn As Integer, indx As Integer
    
    indx = 0    'Initialise the index value.
    
    p_cnt = PP_pgs.Count                'Count the pages.
    If p_cnt > 0 Then
        For j = 1 To p_cnt                  'Check each page,
            pnn = PP_pgs(j).page_number(0)  'storing the LOCAL page number.
            If pnn = pn Then                'Compare this with the target number.
                indx = j                    'Store the index, if found, and
                j = p_cnt                   'terminate the loop.
            End If
        Next
    End If
    
    If indx = 0 Then 'there is an error.
        MsgBox "Page " + CStr(pn) + " isn't indexed!", vbOKOnly + vbCritical, "'find_index' ERROR"
        canceled = True
        find_index = 0
    Else
        find_index = indx
    End If
    
End Property

'Routine to return the current LOCAL page number.
'Function name: get_local_page

Public Function get_local_page()

    get_local_page = p_n
    
End Function

'Routine to return the current Pages collection object.
'Property name: page_col() As Collection

Public Property Get page_col() As Collection

    Set page_col = PP_pgs
    
End Property

'Routine to select a printer.
'The PAPER SIZE and ORIENTATION can't be changed as the text is already drawn.
'Only the dead space is recalculated as a different printer may be selected.
'Procedure name:    select_printer

Public Sub select_printer()

    Dim ppp As PPpage
    Dim indx, j As Integer
    Dim flgg As String
    
    canceled = False            'Clear the 'CANCELed' flag.

    flgg = cdlPDDisablePrintToFile Or cdlPDHidePrintToFile Or cdlPDPrintSetup
    
    cdc.ShowPrinter frm.hWnd, flgg   'Display the 'Print' window.
    
    If Not canceled Then
        Printer.Orientation = p_orient  'Don't allow the orientation and size
        Printer.PaperSize = p_size      'to be altered at this stage.
        
        PgWidth = Printer.Width / p_con
        PgHeight = Printer.Height / p_con

        LRgap = (PgWidth - Printer.ScaleWidth) / 2
        TBgap = (PgHeight - Printer.ScaleHeight) / 2
        parm(9) = LRgap: parm(10) = TBgap
        
        indx = find_index((p_n))     'Find the collection index for this page.
    
        If indx > 0 Then
            Set ppp = PP_pgs.Item(indx) 'Select the page in the collection,
            For j = 9 To 10
                ppp.set_sizes j, parm(j)    'Transfer the Printer dead space values.
            Next
        End If
    End If
   
End Sub

'Routine to select a printer. This version also sets the Excel workbook printer.
'Procedure name:    select_Excel_printer

Public Sub select_Excel_printer()

    Dim flgg, p_nme, p_def As String
    
    canceled = False            'Clear the 'CANCELed' flag.

    flgg = cdlPDDisablePrintToFile Or cdlPDHidePrintToFile Or cdlPDPrintSetup
    
    cdc.ShowPrinter frm.hWnd, flgg   'display the 'Print' window.
    
    If Not canceled Then
        Printer.Orientation = p_orient  'Don't allow the orientation and size
        Printer.PaperSize = p_size      'to be altered at this stage.
        
        p_nme = Printer.DeviceName              'Get the printer name and use it to
        p_def = get_printer_string(p_nme)       'define the Excel printer string which is
        wbk.Application.ActivePrinter = p_def   'used for sheet printing.
        'NB 'wbk' is an ExcelWorkbook class object, defined as 'Public' in the main program.
    End If
   
End Sub

'Routine to get a suitable printer string for Excel.
'Function name: get_printer_string(p_nm)
'Parameter:     p_nm;   the printer name.

Private Function get_printer_string(p_nm)

    Dim p As Printer
    Dim p_port As String
    
    For Each p In Printers              'Check each printer and,
        If p.DeviceName = p_nm Then     'when the default is found,
            p_port = p.Port             'store the port name.
            Exit For
        End If
    Next
    
    get_printer_string = p_nm + " on " + p_port  'Return the printer and port.
    
End Function

'Routine to change the default page setup.
'Procedure name:    page_setup

Public Sub page_setup()

    Dim flgg As String
    
    canceled = False    'Clear the CANCELed flag.

    flgg = units(1, unit) Or cdlPSDDisablePrinter  'Factor = units(0, unit) twips/inch.
 
    cdc.ShowPageSetupDlg frm.hWnd, flgg  'Show the Page setup window.
        
    If Not canceled Then
        cdc.PageData p_data()   'Read the modified page data.
 
        p_con = units(0, unit)
        
        check_paper_size        'Re-set margins, etc.
    End If
    
End Sub

'Routine to get the font data via the 'ShowFont' window.
'Procedure name:    font_setup

Public Sub font_setup()

    Dim flgg As String
    Dim j As Integer
    
    flgg = cdlCFBoth Or cdlCFEffects Or cdlCFForceFontExist Or cdlCFInittoLogFontStruct _
            Or cdlCFLimitSize

    f_parm(0) = cdc.ShowFont(frm.hWnd, flgg) 'Return the selected font title. NB this has NOT been applied.
    
    For j = 1 To 7
        f_parm(j) = cdc.FontData(j) 'Store all the font data.
    Next
    
End Sub

'Routine to define a font item from the defaults.
'Function name: font_dta(opt)
'Parametere:    opt, the option, 0 = name, 1 = size, 2 = Bold, 3 = Italic,
'                                4 = Underline, 5 = Strikethrough,
'                                6 = Text colour, (Forecolor), 7 = Weight, (Not needed).

Public Function font_dta(opt As Integer)

    If opt >= 0 And opt <= 7 Then
        font_dta = f_parm(opt)
    Else
        MsgBox "Options must be in the range 0 to 7", vbOKOnly + vbCritical, "'PPpages:page_data' ERROR"
        canceled = True
        font_dta = 0
    End If
    
End Function

'Error routine to report the various errors.
'Procedure name:    error_report(e_num, f_nm, r_nme)
'Parameter:         e_num;  the error number,
'                   f_nm;  the file name, (where applicable),
'                   r_nme; the routine name.

Private Sub error_report(e_num, f_nm, r_nme)

    Dim txt As String
    
    Select Case e_num
        Case 53 'is 'File not found'.
            txt = "File '" + f_nm + "' doesn't exist."
        Case Else
            txt = "Error # " & Str(Err.Number) & " was generated by " _
            & vbCrLf & Err.Source & vbCrLf & Err.Description
    End Select
    
    MsgBox txt, vbOKOnly + vbCritical, r_nme + " ERROR"
        
    Err.Clear       'Clear the error object.

    canceled = True 'Set the CANCELed flag.
    
End Sub

